
%%                     FRIENDLY ARTIFICIAL INTELLIGENCE OPERATOR light 1.1
%                                                                 for DiVA 
%   . contains f_communicate
%   . contains f_replaceString

% Performs various tasks

%                                                Fabio Crameri, 04.01.2020
%% NOTES

function [fAIo] = f_AIoDiVA(fAIo,Task,SAVE)

%% DEFAULTS
if ~exist('Task','var');            error('no task specified for fAIo...'); 	end
if ~exist('fAIo','var');            fAIo.app = 'na';                            end
if ~isfield(fAIo,'app');            fAIo.app = 'na';                            end
if ~isfield(fAIo,'appVersion');  	fAIo.appVersion = NaN;                      end
if ~isfield(fAIo,'appDate');        fAIo.appDate = 'na';                        end
if ~isfield(fAIo,'off');            fAIo.off = false;                           end
if ~exist('SWITCH','var');          SWITCH.dummy = NaN;                         end
if ~isfield(SWITCH,'Verbose');      SWITCH.Verbose = false;                  	end
if fAIo.off; return; end
if ispc; slash = '\'; else; slash = '/'; end
handleCache             = true;
versionControl          = true;
communicate             = true;

SAVE.urlString          = 'http://www.fabiocrameri.ch/diva';               	%Software webpage
% SAVE.urlFileString      = 'http://www.fabiocrameri.ch/resources/DiVA.zip'; 	%file


if strcmp(fAIo.app,'na') || isnan(fAIo.appVersion) || strcmp(fAIo.appDate,'na')
    %DELETE AND DOWNLOAD NEW VERSION................
end

SAVE.app            = fAIo.app;
SAVE.appVersion     = fAIo.appVersion;
SWITCH.verbose      = false;
numDaysToPause      = 1;

%% CHECK FOR USER AND HOST NAME (might need java)
try
    userName                = getenv('USER');
    if isempty(userName); userName = getenv('USERNAME'); end %try other methods
    if isempty(userName); userName = getenv('LOGNAME'); end
    if isempty(userName); userName = char(java.lang.System.getProperty('user.name')); end %needs Java
catch errorMe
    if SWITCH.verbose; warning(errorMe.message); end
    userName                = 'unknown';
end
try
    hostName                = getenv('COMPUTERNAME'); %only windows
    if isempty(hostName);   [~,hostName] = system('hostname'); end
    if isempty(hostName);   hostName = char(java.net.InetAddress.getLocalHost.getHostName); end %needs Java
catch errorMe
    if SWITCH.verbose; warning(errorMe.message); end
    hostName                = 'unknown';
end

%% CHECK FOR ORIGINAL RUNNING SCRIPT (PARFILE)
mFilesInUse                 = dbstack('-completenames');
currentParfileName          = mFilesInUse(end,1).name;      %original file (i.e., parfile)
currentParfileNameEndix     = [currentParfileName,'.m'];    %original file (i.e., parfile) with file ending
currentParfile              = mFilesInUse(end,1).file;      %original file (i.e., parfile) full file stem
currentParfilePath          = currentParfile(1,1:max(strfind(currentParfile,slash))); %path to original file

currentDiVAUpdateName       = mFilesInUse(2,1).name;      %update file (i.e., parfile)
currentDiVAUpdateNameEndix	= [currentDiVAUpdateName,'.m'];    %update file (i.e., f_DiVAupdate) with file ending
currentDiVAUpdate           = mFilesInUse(2,1).file;      %original file (i.e., f_DiVAupdate) full file stem
currentDiVAUpdatePath       = currentDiVAUpdate(1,1:max(strfind(currentDiVAUpdate,slash))); %path to update file

cacheFile                   = [currentDiVAUpdatePath,'f_DiVAupdate.m'];

%% SOLVING TASK I
numDaysToPauseCommunication = numDaysToPause;	%<< specify communication interval here in [days]
if communicate
    if datetime(date)>=datetime(fAIo.appDate)+numDaysToPauseCommunication
        emailTo       	= 'fabiocrameri@me.com';
        subject         = [userName,'''s DiVA Usage'];
        message         = [
            {userName}; %these brackets are somehow needed for the whole array to work...
            strcat('(',hostName,')');
            [' '];
            ['   Current Version:       ',SAVE.app,' ',num2str(SAVE.appVersion)];
            [' '];
            [' ']; %needed to prevent overlapp of attachement and text
            [' '];
            [' '];
            ];
        
        [COMMUNICATE] = f_communicate(emailTo,subject,message);
    else
        handleCache = false; %no need to update cache
    end
end



%% SOLVING TASK II
numDaysBetweenChecks = numDaysToPause;	%<< specify version check interval here in [days]
if versionControl
    if datetime(date)>=datetime(fAIo.appDate)+numDaysBetweenChecks
        versionControlSuccessful = true; %default
        %% CHECK FOR CURRENT IMAGES2MOVIE VERSION
        %Assumes that there is only ONE 'Version X.X' or 'Version X.XX' string
        %on the whole webpage!
        idx = NaN; PreString = ''; textString = '';
        try
            textString = urlread(SAVE.urlString,'Timeout',5); %5 sec timeout
            WebCrawlSuccessful = true;
        catch errorMe
            if SWITCH.Verbose; warning(errorMe.message); end
            WebCrawlSuccessful = false; versionControlSuccessful = false;
        end
        if WebCrawlSuccessful
            PreString = 'dv-vn';   %string to look for <<<<<<<<<<<<<<<<<<<<<
            idx = strfind(textString,PreString);
            if size(idx,1)>1 || size(idx,2)>1
                if SWITCH.verbose; warning('More than one ''dv-vn'' string found!'); end
                idx     = idx(1,1); %take first one
            end
            if isempty(idx) %string not found - look for different string <<<<<
                PreString = 'DV-VN';
                idx = strfind(textString,PreString);
            end
            if isempty(idx) %string not found
                if SWITCH.verbose; warning('Version string not found!'); end
                versionControlSuccessful = false;
            end
        end
        if versionControlSuccessful
            % read StagLab version
            idxStart    = idx+length(PreString);
            idxEnd      = idx+length(PreString)+3;
            version     = textString(idxStart:idxEnd);
            version     = strrep(version,'&n',''); %remove possible 2 empty spaces at the end
            version     = strrep(version,'&',''); %remove possible empty space at the end
            version     = strrep(version,')',''); %remove possible brackets at the end
            version     = strrep(version,'(',''); %remove possible brackets at the beginning
            SAVE.urlFileString      = ['http://www.fabiocrameri.ch/resources/DiVA',version,'.zip']; 	%file

            try
                % compare available version with current version
                versionNum  = str2double(version);
                % if new version available: let the user know
                if versionNum>SAVE.appVersion
                    disp(' ');
                    disp([' *** New DiVA ',num2str(versionNum),' is available <a href="http://www.fabiocrameri.ch/software">here</a>! ***']);
                    disp(' ');
                    
                    CACHE.newVersionAvailable   = true; %remember that a new version is available
                    
                    %ask to install new version
                    b=questdlg(['Do you want to download the new DiVA ',num2str(versionNum),'?'],...
                        ['DiVA ',num2str(versionNum),' is available!'], ...
                        'Yes','Read more','Remind me later','Yes');
                    switch b
                        case 'Yes'
                            %open browser and start download
                            %web(SAVE.urlFileString,'-browser') %use system's default browser
                            %web(SAVE.urlString,'-notoolbar','-noaddressbox','-new');
                            web(SAVE.urlFileString);
                            %finish current StagLab task
                            fAIo.status  	= 'fAIo has aborted current task due to DiVA update.';
                            
                        case 'Read more'
                            %open browser
                            web(SAVE.urlString,'-notoolbar')
                            %finish current StagLab task
                            fAIo.status  	= 'fAIo has aborted current task due to possible DiVA update.';
                            
                        case 'Remind me later'
                            %continue
                    end
                else %current version is up-to-date
                    CACHE.newVersionAvailable   = false;
                end
            catch errorMe
                if SWITCH.Verbose; warning(errorMe.message); end
                handleCache = false; %no need to update cache
            end
        end
    else
        handleCache = false; %no need to update cache
    end
end



%% SOLVING TASKS III
if handleCache
    % remember successful version check date
    fAIo.appDate    = date;   %update Cache variable
                
    %% UPDATE DATE STAMP ON FILE
    %for full line replacements add *** at the end of the string:
    Replacements    = {'fAIo.appDate***',          	['fAIo.appDate=''',date,''';']};
    %update currently used parfile
    InputFile       = cacheFile;
    OutputFile      = InputFile; %overwrites file       %NEEDS TO BE FULL FILE STEM!
    
    SearchString    = Replacements{1,1};
    ReplaceString   = Replacements{1,2};
    f_replaceString(InputFile,OutputFile,SearchString,ReplaceString) %needs char input
end

end


function [COMMUNICATE] = f_communicate(emailTo,subject,message)
%% INPUT
%f_AIo's Gmail account
mailAddress   	= 'faiocommunication@gmail.com';
password        = 'useWisely';

%% SETUP SMTP
setpref('Internet','E_mail',mailAddress);
setpref('Internet','SMTP_Server','smtp.gmail.com');
setpref('Internet','SMTP_Username',mailAddress);
setpref('Internet','SMTP_Password',password);

%% SETUP AUTHENTIFICATION
props = java.lang.System.getProperties;
props.setProperty('mail.smtp.auth','true');
props.setProperty('mail.smtp.socketFactory.class','javax.net.ssl.SSLSocketFactory');
props.setProperty('mail.smtp.socketFactory.port','465');

%% SENDING E-MAIL (given internet connection)
try
    sendmail(emailTo,subject,message)
    COMMUNICATE.successful  = true;
    COMMUNICATE.lastDate    = date;
catch
    COMMUNICATE.successful  = false;
    COMMUNICATE.lastDate    = NaN;
end

end




function [] = f_replaceString(InputFile,OutputFile,SearchString,ReplaceString)
%% change data [e.g. initial conditions] in model file
% InputFile - string
% OutputFile - string
% SearchString - string/char
% ReplaceString - string/char

% read whole model file data into cell array
fid = fopen(InputFile);
data = textscan(fid,'%s','Delimiter','\n','CollectOutput',true,'WhiteSpace','');
fclose(fid);

% modify the cell array
% find the line where changes need to be applied and insert new data
for i=1:length(data{1})
    if strcmp(SearchString(end-2:end),'***') %replace whole line
        foundStringPosition = strfind(data{1}{i},SearchString(1:end-3));
    else
        foundStringPosition = strfind(data{1}{i},SearchString);
    end
    % search for this string in the array
    if ~isempty(foundStringPosition)
        if strcmp(SearchString(end-2:end),'***') %replace whole line
            ReplacementLine = ReplaceString;
        else %replace string in line
            ReplacementLine = strrep(data{1}{i},SearchString,ReplaceString);
        end
        data{1}{i} = ReplacementLine; % replace with this string
    end
end

% write the modified cell array into the text file
fid = fopen(OutputFile,'w');
for i=1:length(data{1})
    fprintf(fid, '%s\n', char(data{1}{i}));
end
fclose(fid);
end
